# Overpass Server Installation and Setup
I found setting up the overpass server to be fairly straightforward. The package is generally called osm-3s (for Server Side Scripting).


#### Online Resources
The main installation page https://wiki.openstreetmap.org/wiki/Overpass_API/Installation

Regional data files (download the appropriate osm.bz2 files) \
https://download.geofabrik.de/north-america.html \
https://download.geofabrik.de/europe.html


#### Quick steps
1. Download the tarball using the installation page instructions
2. make install to build the executables.
3. Download the appropriate regional data osm.bz2 files from geofabrik server.
4. Load the data files into the database.
5. Create the start script and service file.



#### System requirements
The main requirement will be space for the database. It is recommended to not use a spinning hard drive.
The recommended method of populating the database is by cloning the public dev server with download_clone.sh
Cloning the dev server has advantages but it will include the entire world, which will consume hundreds of gigabytes.

I decided to only download and populate selected regions using init_osm3s.sh and and osm.bz2 files downloaded from the geofabrik server.
These were my results after loading region files for US and Canada
| region       | osm.bz2 file size | database size |
|--------------|-------------------|---------------|
| canada       |         4.30GB    |      36.53GB  |
| us-midwest   |         2.45GB    |      48.27GB  |
| us-northeast |         1.69GB    |      52.04GB  |
| us-pacific   |         0.25GB    |      53.23GB  |
| us-south     |         4.04GB    |      60.67GB  |
| us-west      |         3.29GB    |      63.79GB  |


| Minimum space requirements    |      |
|-------------------------------|------|
| database                      |    64GB|
| largest osm.bz2 download file |     5GB|
| executables & code            |     0.07GB|
| total minimum                 |   70GB |

The system could be housed in as little as 70GB. However, 140GB would allow for building an offline copy of the database to apply updates.
Operating without offline rebuilds would force the system to be shutdown for around 5 hours when a database update is required.


#### Install and Build
It is recommended to have a separate user with no special permissions to host OSM. Create a user if one is not available. I created an osm user for this.

Verify the prerequisites are installed (C++, make, expat zlib)
Follow the installation page to download and extract the osm-3s tar. I used Option 1 tarball

Set environment variables \
EXEC_DIR - Where the executable go. It will create bin and cgi-bin directories here. \
DB_DIR - directory where the database files will go.

Then use
```
make install
```
I found it to work without issues. You could also do the build on a similar system and copy the resulting executables to avoid source code clutter on the server.

The Static Usage section describes verifying the executables with
```
$EXEC_DIR/bin/osm3s_query --db-dir=$DB_DIR
```

#### Populating the database
As mentioned, I used regional download files and the init_osm.sh script to conserve space. Populating the US & Canada data took around 5.5 hours on my system.
I did not try cloning the database since we do not need the entire world. The main advantage of cloning the database is that an automatic incremental update can scheduled to periodically download changes and apply them.
I feel we do not need the very latest data so the overhead of maintaining constantly updated copy of the entire world is not justified. I feel the periodic downloads of the reqgions and rebuilding of the database will be sufficient for our purposes.

I found that we do not need metadata or attic data. These are both used to track authors and previous versions of the data.
Use commands like this to populate the database.
```
nohup ../src/bin/init_osm3s.sh us-latest.osm.bz2 $DB_DIR $EXEC_DIR &
```

#### Running the dispatcher
The dispatcher is the database server for the system. It is recommended to start it as a background process as follows. \
The --osm-base tells it to serve basic queries.\
There is an "area" option for running area queries. This would be run as a separate process but it is not required.
```
nohup "$EXEC_DIR/bin/dispatcher" --osm-base "--db-dir=$DB_DIR" >> ~/osm3s_dispatcher.log 2>&1 &
```
The dispatcher can be run from the command line for testing. It can be checked using the query script without a --db-dir argument.
```
$EXEC_DIR/bin/osm3s_query
```
Enter a sample query like this followed by control-D
```
<query type="node"><bbox-query n="30" s="20" w="-97" e="-87"/><has-kv k="amenity" v="pub"/></query><print/>
```

The query without a --db-dir parameter will only succeed if the dispatcher is running.


#### Stopping the dispatcher
The preferred way to stop the dispatcher is with 
```
$EXEC_DIR/bin/dispatcher --terminate
```
Killing the process will leave orphaned socket and other files that will prevent it restarting.


#### Web front end
The web API front end is provided by a standard CGI program. The installation web page has examples of setting it up with apache and nginx. I ran both apache and nginx with no issues. Some changes will be required to work it into an existing site configuration but that will be system specific.

The nginx config will look something like this.
```
# cat overpass.conf
server {
    listen 80;
    location /api/ {
        alias /home/osm/overpass/bin/cgi-bin/;
        #gzip on;
        #gzip_types application/json application/osm3s+xml;
        
        # set the minimum length that will be compressed
        #gzip_min_length 1000; # in bytes
        
        # Fastcgi socket
        fastcgi_pass  unix:/var/run/fcgiwrap.socket;
        
        # Fastcgi parameters, include the standard ones
        include /etc/nginx/fastcgi_params;
        
        # Adjust non standard fcgi parameters
        fastcgi_param SCRIPT_FILENAME  $request_filename;
    }
}
```

#### Automatically running the dispatcher on startup
There are numerous warnings against running the dispatcher as root or trying to start it with systemd. The recommended method to auto-start it is using an @reboot crontab entry for the osm user.
The cron @reboot approach works but it lacks controlled policies for logging or shutdown.

Our consensus is that running it as a service is the preferred method. It provides well defined and well understood startup, shutdown and logging policies. The service can be configured to run as the desired user since running it as root is genuinely not desired.


Create a start_dispatcher.sh script in the osm user bin directory. This handles killing any dispathcer that may be left running an cleaning up orphaned files before starting the dispatcher.
```
#!/usr/bin/env bash

# Adapted from reboot.sh by tvolle@solartechnology.com
# Attempt to stop any running dispatchers and clean up orphaned files prior to starting.
#
# This script does not return until the dispatcher is stopped. It is intended to be run by systemd

#check for the appropriate user
if [[ `id -nu` != "overpass" ]]; then
   echo "current user is "`id -nu`"... this script must be run as osm user, exiting.."
   exit 1
fi

EXEC_DIR=~/bin
DB_DIR=~/db

echo "OSM dispatcher startup script"
echo "EXEC_DIR="`echo $EXEC_DIR`
echo "DB_DIR="`echo $DB_DIR`

# check for a dispatcher running and try to terminate
if test `ps -ef | grep $EXEC_DIR/dispatcher | grep -v grep | wc -l` -gt 0
  then
  echo "found running dispatcher, stopping "`ps -f | grep $EXEC_DIR/dispatcher | grep -v grep`
  $EXEC_DIR/dispatcher --terminate
fi

# if any are still running try to force kill them
ps -ef | grep $EXEC_DIR/dispatcher | grep -v grep | while
  read u p x
  test -n "$p"
do
    echo "Found unexpected dispatchers still running, trying to force kill"
    echo $u" "$p" "$x
    kill -9 $p
done


# look for lock files and remove - a force kill of the dispatcher will leave these behind
if test -e $DB_DIR/osm3s*_osm_base -o -e /dev/shm/osm3s*_osm_base
  then
  echo "found existing lockfiles... this indicates a bad shutdown of the previous dispatcher. Cleaning up."
  rm -f $DB_DIR/osm3s*_osm_base
  rm -f /dev/shm/osm3s*_osm_base
fi

echo "starting dispatcher"
## These limits can be used but require more research.
## I found the rate limit 2 causes an excessive slow down.
#$EXEC_DIR/dispatcher --osm-base --meta --rate-limit=2 --space=10737418240 --db-dir="$DB_DIR"
$EXEC_DIR/dispatcher --osm-base --db-dir="$DB_DIR"
```


Create a file /etc/systemd/system/overpass-dispatcher.service
Change the user and group as appropriate.
```
[Unit]
Description=Open Street Map Overpass Dispatcher
DefaultDependencies=no
After=network.target

[Service]
Type=simple
User=osm
Group=osm
ExecStart=/home/overpass/lbin/start_dispatcher.sh
ExecStop=/home/overpass/bin/dispatcher --terminate
TimeoutStartSec=0
RemainAfterExit=yes

[Install]
WantedBy=default.target
```

The service can then be controlled with systemctl as overpass-dispatcher

```
# systemctl status overpass-dispatcher
● overpass-dispatcher.service - Open Street Map Overpass Dispatcher
     Loaded: loaded (/etc/systemd/system/overpass-dispatcher.service; enabled; vendor preset: enabled)
     Active: active (running) since Wed 2021-01-06 11:38:28 EST; 19min ago
   Main PID: 88741 (bash)
      Tasks: 2 (limit: 77014)
     Memory: 10.6M
     CGroup: /system.slice/overpass-dispatcher.service
             ├─88741 bash /home/osm/overpass/bin/bin/start_dispatcher.sh
             └─88767 /home/osm/overpass/bin/bin/dispatcher --osm-base --meta --db-dir=/home/osm/overpass/db

Jan 06 11:38:28 solartechv systemd[1]: Started Open Street Map Overpass Dispatcher.
Jan 06 11:38:28 solartechv start_dispatcher.sh[88741]: OSM dispatcher startup script
Jan 06 11:38:28 solartechv start_dispatcher.sh[88741]: EXEC_DIR=/home/osm/overpass/bin/bin
Jan 06 11:38:28 solartechv start_dispatcher.sh[88741]: DB_DIR=/home/osm/overpass/db
Jan 06 11:38:28 solartechv start_dispatcher.sh[88741]: found existing lockfiles... this indicates a bad shutdown of the previous dispatcher. Cleaning up.
Jan 06 11:38:28 solartechv start_dispatcher.sh[88741]: starting dispatcher
# 
```


#### Automating database updates
We do not need the overhead of continuous updates. However, periodic updates would be useful to keep the data relatively current. It would desirable to make the updates automatic since they are time consuming and tedious.

The following script can be used to initially populate the database and to periodically update it. The last updte timestamp is checked against the available versions of the files online. If any file is found to be stale then the entire database is rebuilt.
It is possible to apply individual files to the database but I found two main drawbacks to this approach.
First, the database seems to continually grow, even when the same files is applies multiple time. So we could expect the database to continue growing and need periodic full rebuilds if we apply individual files to it.
The database must also be offline during the loading process. Therefore it seems best to simply rebuild the entire database from scratch when any files are found to be stale.
Ideally, the database could be rebuilt in a separate directory while the previous database remains onlint. Then the dispatcher can be stopped and the new database swapped in with minimal downtime.

The exact mechanism and frequency of updates is yet to be determined. The following script will check the file timestamps and rebuild as necessary into a separate directory. It should be run in cron as the overpass user.

```
#!/usr/bin/env bash

TARGETS="http://download.geofabrik.de/north-america/canada-latest.osm.bz2 http://download.geofabrik.de/north-america/us-midwest-latest.osm.bz2 http://download.geofabrik.de/north-america/us-northeast-latest.osm.bz2 http://download.geofabrik.de/north-america/us-pacific-latest.osm.bz2 http://download.geofabrik.de/north-america/us-south-latest.osm.bz2 http://download.geofabrik.de/north-america/us-west-latest.osm.bz2"

EXEC_DIR=~/bin

DB_DIR=~/new-db

LAST_DOWNLOAD_TIMESTAMP_DIR=~/last-download-timestamps

IN_PROGRESS_DIR=~/in-progress


##########################################################
# Get last update time from the http headers on a resource
#
get_remote_timestamp () {
  local url=$1
  declare -A headers

  while
    read x
  do
    echo "$x"
    key=${x%%:*}
    val=${x#*:}
    val="${val#"${val%%[![:space:]]*}"}"
    if [[ -n "$key" ]]; then
      headers[$key]="$val"
    fi
  done << EOF
$(curl -I $url 2>/dev/null)
EOF
  local lastUpdate="${headers[Last-Modified]}"
  touch -d "$lastUpdate" last_update
  if [[ $? -ne 0 || -z "$lastUpdate" ]]; then
    echo "failed to get last update time from $url"
    exit 1
  fi
}


##############################
# Ensure directories are valid
#
if [[ ! -e $LAST_DOWNLOAD_TIMESTAMP_DIR ]]; then
  mkdir $LAST_DOWNLOAD_TIMESTAMP_DIR
fi
if [[ ! -e $IN_PROGRESS_DIR ]]; then
  mkdir $IN_PROGRESS_DIR
fi
rm -f "$IN_PROGRESS_DIR/*"


#########################################################
# Check for files that are newer than our current imports
#
echo -e "\n\n\nstarting imports "`date`
STALE_TARGET_COUNT=0

for TARGET in $TARGETS
do
  echo -e "\n\n\nchecking last update time on "$TARGET
  get_remote_timestamp $TARGET
  file_name=$LAST_DOWNLOAD_TIMESTAMP_DIR/`basename "$TARGET"`

  if [[ ! -e $file_name || `find -newer $file_name -name last_update | wc -l` -gt 0 ]]; then
    tfn=`basename "$TARGET"`
    echo "stale $tfn"
    ((STALE_TARGET_COUNT++))
  fi
done
if [[ $STALE_TARGET_COUNT -lt 1 ]]; then
  echo -e "\n\n\nall targets up to date"
  exit 0
fi
echo -e "\n\n\nfound $STALE_TARGET_COUNT stale targets"


#############################################################################################################################
# Set the list of targets to reload. My tests have shown that loading into an existing database causes it to continually grow
# so I rebuild by loading all targets into an empty dir if anything is stale.
# 
# We could only import the stale files into an existing DB but we would have to monitor the total size to ensure it is not
# getting too big.
# 
TARGETS_TO_RELOAD=$TARGETS

# ensure an empty database dir
if [[ ! -e $DB_DIR ]]; then
  mkdir $DB_DIR
fi
rm -f $DB_DIR/*

for TARGET in $TARGETS_TO_RELOAD
do
  echo -e "\n\n\ndownloading $TARGET "`date`
  FILE_NAME=`basename "$TARGET"`
  rm -f "$FILE_NAME"
  wget -nv $TARGET
  if [[ $? -ne 0 || ! -e "$FILE_NAME" ]]; then
    echo -e "\n\nerror downloading $TARGET"
    echo quitting
    exit 1
  fi
  echo "done downloading $FILE_NAME "`date`
  ls -l $FILE_NAME

  touch -r $FILE_NAME $IN_PROGRESS_DIR/$FILE_NAME

# I found that the compression setting actually made the database a bit larger.
#  $EXEC_DIR/bin/init_osm3s.sh $FILE_NAME $DB_DIR $EXEC_DIR --compression-method=gz
  $EXEC_DIR/bin/init_osm3s.sh $FILE_NAME $DB_DIR $EXEC_DIR --compression-method=gz
  if [[ $? -ne 0 ]]; then
    echo "non zero return from init_osm3s, exit="$?
    rm $FILE_NAME
    exit 1
  fi
  rm $FILE_NAME
  echo "DB size "`du -b $DB_DIR`
done

mv $IN_PROGRESS_DIR/* $LAST_DOWNLOAD_TIMESTAMP_DIR

echo -e "\n\n\ndone with imports "`date`
```





