# THE MONITORING APP - DESIGN & USE

_Initial Note: for a direct path/link to any of the classes mentioned below, please see the list in the **"Classes Paths"** senction_.
## Overview
This document will give you an overview of the functionalities and of the design of the Monitoring Application designed to send out notification e-mails to a specific group of SolarTech engineers when an error-conditions is met/found and needs to be addressed/reviewed.

## How It Works
The purpose of the Monitoring Application is to run server-side along with our main Command-Center and/or SmartZone app(s) and provide active feedback to the SolarTech team about execution-record related errors and server-data used to track the servers' performance and ongoing functionality.

At the time of completion of the initial version of the Monitoring App (v 1.0), the app is divided into 2 main tracking/monitoring areas: **execution-record monitoring**, which concentrates on tracking and analyzing `ExecutionError`s found within any `ExecutionRecord`, and the **server-data monitoring** that periodically pulls each server's performance-related values and looks for inconsistencies and/or dangerous conditions/values.

The Monitoring Application stores lightly-processed execution record and server diagnostic data in a local MongoDB database.  Queries are run against this data each time it is updated to identify and update alerts.  Active and historical alerts are also tracked in the database.

#### Execution-Record Monitoring
The portion of the application that takes care of monitoring all `ExecutionError` relies on an initial request, sent out to the server, to add the app's remote-connection instance to a list of remote-connections that need/want to receive all and any `ExecutionRecord` produced. Once this initial connection and addition to the list have been completed, the app simply waits to receive the reports.

When the app receives an `ExecutionRecord`, it stores it in the database and updates alerts for the scenario in the `ExecutionRecord`.  If there are new alerts for the condition, they are added to the database, and an alert notification email is sent out to all addresses in the NotificationRecipients collection in the database.  Any existing alerts in the database which are no longer active as a result of the new `ExecutionRecord` are inactivated in the database, and a clear noticiation is sent out.

The current alert conditions we look for in each `ExecutionRecord` and each related `ExecutionError` are the following:
 - _**5 times in a row**_: this condition checks for the occurrence of an `ExecutionError` 5 times in a row. This condition gets reset as soon as that same `ExecutionRecord` (about the same scenario) is received and the specific `ExecutionError` is not reported within it.
 - _**10 times within a 30 minutes time-frame**_: this condition checks for the occurrence of an `ExecutionError` 10 times (or more) within a 30 minutes (or less) window. This condition gets reset when the number of occurrences of the error in the last 30 minutes drops below 10.

#### Server-Data Monitoring
The portion of the application that takes care of monitoring server-data relies on a **timed-task** that runs once every minute (for now) and pulls the monitoring data associated with each provided server.

Once the data for each server has been received and parsed through, it is added to the database, and queries are run against it to identify new alerts.  If there are new alerts for any conditions, they are added to the database, and an alert notification email is sent out to all addresses in the NotificationRecipients collection in the database.  Any existing alerts in the database which are no longer active are inactivated in the database, and a clear notification is sent out.

The current `ServerCondition` types we look for in each server-data pull are the following:
 - _**connected active units decreased in absence of restart**_: this condition checks for a conspicuous difference between the number of currently `connected-assigned-units` across all servers and the average value from the last 24 hours. This condition gets reset as soon as the difference between values drops below the set "conspicuous difference" (10% of the average) ceiling.
 - _**low available heap**_: this condition checks for an `available-heap` value of less than 2.0 Gb in combination with a `time-since-start` value of 30 minutes (or more). This condition gets reset when the `available-heap` raises above the 2.0 Gb minimum or when the server is restarted (which would be indicated by a `time-since-start` value of fewer than 30 minutes).
 - _**mongo-time value above 10 seconds**_: this condition checks for a `mongo-query-time` and/or a `mongo-save-time` value(s) of more than 10 seconds at any time. This condition gets reset as soon as both mongo-time related values are found to be less than 10 seconds.

### Port Monitoring
The portion of the application that tests connections to TCP ports on a **timed-task** that runs every five minutes.

The ports to be monitored are configured through the `Ports` database collection, which has the following format:

  { "host": "diphda.sthosts.net", "port": 1039, "portName": "VPN piercer" }

After a port is tested on schedule, a `PortDiagnostics` record is stored for each host and port combination tested.  The presence of an `error` key indicates that the connection test failed, and `error` contains an error message.

`PortCondition` checks the most recent `PortDiagnostics` record for each host and port and creates an alert if one is present.  If there are new alerts for the condition, they are added to the database, and an alert notification email is sent out to all addresses in the NotificationRecipients collection in the database.  Any existing alerts in the database which are no longer active are inactivated in the database, and a clear notification is sent out.

## Design

The monitoring app has been designed as a separate application that can run as a stand-alone app, but, obviously, without the actual server for the main SolarTech app(s) running, the monitoring app won't be able to connect and perform any monitoring.

The main class related to this application is `SmartzoneMonitor.java`, which takes care of instantiating the `ExecutionMonitor` instances and runs periodic tasks once per minute including:

 - Fetching and analyzing server diagnostic data
 - Clearing old execution alerts for scenarios which are no longer running
 - Cleaning up old execution record, server diagnostic, and alerts from the database
 - Sending a daily report every morning

Server alerts are maintained by the `ServerAlerts` class which checks the server diagnostics in the database for new alert conditions.  The `SmartzoneMonitor` triggers the checks periodically after each new server diagnostics record is added to the database.  Active and historical alerts are maintained in the `ServerAlerts` collection in the database.  The `ServerAlerts` class sends alert notifications when new alerts are added and when existing alerts are inactivated.

Execution alerts are maintained by the `ExecutionAlerts` class which checks the execution records for specific scenarios for new alert conditions.  These checks are triggered by the `ExecutionMonitor` class after each new execution record for a scenario is added to the database.  Active and historical alerts are maintained in the `ExecutionAlerts` collection in the database.  The `ExecutionAlerts` class sends alert notifications when new alerts are added and when existing alerts are inactivated.

Both Alerts classes track alerts based on sets of `ExecutionCondition` or `ServerCondition` objects which specify database queries and transforms to identify new alerts.  Each object of classes which implement a `Condition` query an associated collection - generally, `ExecutionCondition` conditions query the `ExecutionRecords` collection, and `ServerCondition` conditions query the `ServerDiagnostics` collection.  The base Condition classes provides methods which apply to the collection an aggregate pipeline specified by its implementors which returns a list of `ExecutionAlert` or `ServerAlert` objects which the Alerts classes add to their corresponding Alerts database collections.

Both Alerts classes add these fields to Alert objects before adding them to the database:
 - "condition": a string that uniquely identifies the condition type (from `Condition.getType()`)
 - "created": the time at which the alert first became active
 - "active": a boolean indicating whether or not the alert is active
 - "updated": the most recent time the alert was still active - this gets touched each time the condition returns the alert

At the time of completion of the initial version of the Monitoring App (v 1.0), there are 2 classes that extend `ExecutionCondition`:
 - `FiveConsecutive.java`
 - `TenInThirty.java`

and 3 that extend `ServerCondition`:
 - `UnitsLow.java`
 - `HeapLow.java`
 - `MongoTimeHigh.java`

`ServerAlerts` uses `ServerEmail` to format and send notification emails.  `ExecutionAlerts` uses `ExecutionEmail` to format and send notification emails.  Both `Email` classes implement a simple template language for formatting email subjects and bodies.

The `Alerts` classes use the `NotificationRecipients` class to query the database for a list of email addresses to which to send notifications.  The email addresses are stored in the `email` field of records in the `NotificationRecipients` collection.

`ExecutionAlerts` ignores notifications for organizations whose names match criteria in the `OrganizationExclusions` table of the database.  See `ExecutionAlerts.isOrganizationExcluded`.

#### Specific Design Notes associated with the Execution-Record Monitoring side of the Application
This monitoring side of the app strongly relies on a connection on port **2001** (the server would be listening on this port for a `ClientConnection` connection - which is set up to do so in the `SolarNetServer.initServer()` method), the initial request to be added to the execution-record "need-to-know" clients list mentioned earlier, and on the reception of those execution-records. The errors are then parsed, stored, counted, and accounted for; a notification is sent out when a condition is met and reset when the reset-parameter(s) of that condition is(are) met.

The initial request is initiated by `ExecutionMonitor` that sends out a `MsgExecutionRecordsMonitoring` message to the `ClientConnection`, and when the `ClientConnection` receives this message it calls the `ClientConnection.addClientToExecutionRecordsMonitoring()` method to add the current `RemoteConnection` (itself - the `ClientConnection`) to the `SolarNetServer.executionRecordsMonitoringClients` list, which again is the list of all remote-connections that want to be notified of all and any execution-record.

The execution-records are sent by the `SolarNetServer.smartzoneExecutionCompleted()` method which is called every time a `ScenarioNodeRoot`'s execution is completed (the `ScenarioNodeRoot.executionFinished()` method is called) and an `ExecutionRecord` is created.

The server this side of the app will connect to is picked from a list of them hard-coded and saved in the `SmartzoneMonitor`'s `SOLARNET_SERVERS` collection; at any time a new one is introduced, the `SOLARNET_SERVERS` list will need to be updated.

#### Specific Design Notes associated with the Server-Data Monitoring side of the Application
This monitoring side of the app strongly relies on the data each server provides when contacted/connected-to on port **2014** (the server would be listening on this port for a `DiagnosticConnection` connection - which is set up to do so in the `SolarNetServer.initServer()` method).

The `main` method that initializes this timer-task-based monitoring instance is in `SmartzoneMonitor`, and it has been designed and configured to read `args` and retrieve data from specific ones. This could easily be modified to look for one more flag to read a specific number of minutes or seconds the server-data monitoring's timer-task needs to be set to if/when needed; at the time of completion of the initial version of the Monitoring App (v 1.0), this option was not necessary so it was not implemented (1 minute seemed reasonable enough at this time).

The servers this side of the app will monitor are hard-coded, and if a new one is introduced, the list of servers the app relays on will need to be updated. The list can be found in the `SmartzoneMonitor.java` class saved as the `SOLARNET_SERVERS` collection.

#### Classes Paths:
 * Classes used throughout the app:

     - [DailyReportEmail.java](~/solartech/java/src/com/solartechnology/monitor/DailyReportEmail.java)
     - [DailyReport.java](~/solartech/java/src/com/solartechnology/monitor/DailyReport.java)
     - [Email.java](~/solartech/java/src/com/solartechnology/monitor/Email.java)
     - [NotificationRecipients.java](~/solartech/java/src/com/solartechnology/monitor/NotificationRecipients.java)
     - [OrganizationExclusions.java](~/solartech/java/src/com/solartechnology/monitor/OrganizationExclusions.java)
     - [OrganizationStatistics.java](~/solartech/java/src/com/solartechnology/monitor/OrganizationStatistics.java)
     - [SmartzoneMonitor.java](~/solartech/java/src/com/solartechnology/monitor/SmartzoneMonitor.java)
     - [StatisticsCollector.java](~/solartech/java/src/com/solartechnology/monitor/StatisticsCollector.java)
     - [Statistics.java](~/solartech/java/src/com/solartechnology/monitor/Statistics.java)
     - [NotificationRecipient.java](~/solartech/java/src/com/solartechnology/monitor/entities/NotificationRecipient.java)

 * Execution-Record Monitoring Specific Classes:

     - [ExecutionAlerts.java](~/solartech/java/src/com/solartechnology/monitor/ExecutionAlerts.java)
     - [ExecutionCondition.java](~/solartech/java/src/com/solartechnology/monitor/ExecutionCondition.java)
     - [ExecutionEmail.java](~/solartech/java/src/com/solartechnology/monitor/ExecutionEmail.java)
     - [ExecutionMonitor.java](~/solartech/java/src/com/solartechnology/monitor/ExecutionMonitor.java)
     - [TenInThirty.java](~/solartech/java/src/com/solartechnology/monitor/conditions/TenInThirty.java)
     - [FiveConsecutive.java](~/solartech/java/src/com/solartechnology/monitor/conditions/FiveConsecutive.java)
     - [OrganizationExclusion.java](~/solartech/java/src/com/solartechnology/monitor/entities/OrganizationExclusion.java)
     - [ExecutionAlert.java](~/solartech/java/src/com/solartechnology/monitor/entities/ExecutionAlert.java)
     - [ExecutionRecord.java](~/solartech/java/src/com/solartechnology/monitor/entities/ExecutionRecord.java)

 * Server-Data Monitoring Specific Classes

     - [ServerAlerts.java](~/solartech/java/src/com/solartechnology/monitor/ServerAlerts.java)
     - [ServerCondition.java](~/solartech/java/src/com/solartechnology/monitor/ServerCondition.java)
     - [ServerDataChanges.java](~/solartech/java/src/com/solartechnology/monitor/ServerDataChanges.java)
     - [ServerDataToCompare.java](~/solartech/java/src/com/solartechnology/monitor/ServerDataToCompare.java)
     - [ServerDiagnostics.java](~/solartech/java/src/com/solartechnology/monitor/ServerDiagnostics.java)
     - [ServerEmail.java](~/solartech/java/src/com/solartechnology/monitor/ServerEmail.java)
     - [ServersDataMonitor.java](~/solartech/java/src/com/solartechnology/monitor/ServersDataMonitor.java)
     - [MongoTimeHigh.java](~/solartech/java/src/com/solartechnology/monitor/conditions/MongoTimeHigh.java)
     - [HeapLow.java](~/solartech/java/src/com/solartechnology/monitor/conditions/HeapLow.java)
     - [UnitsLow.java](~/solartech/java/src/com/solartechnology/monitor/conditions/UnitsLow.java)
     - [ServerAlert.java](~/solartech/java/src/com/solartechnology/monitor/entities/ServerAlert.java)
     - [ServerDiagnostic.java](~/solartech/java/src/com/solartechnology/monitor/entities/ServerDiagnostic.java)

#### How to Add a New Condition (for either the Execution-Record or the Server-Data side of the Application)
To add a new condition, a new class will need to be created in that will need to extend the `ExecutionCondition` or `ServerCondition` class.  The new class should be created in the ~/solartech/java/src/com/solartechnology/monitor/conditions directory.

The new class needs to provide some information to the Condition constructor:
 - collectionClass: the class corresponding to the database collection that the condition queries - generally, this will be `ExecutionRecord.class` for `ExecutionCondition` and `ServerDiagnostic.class` for `ServerCondition`.
 - type: this is the type returned by `getType()`, used by the Alerts classes to connect alerts in the database to this type of Condition
 - description and subject: these can be used in notification emails, see `ExecutionEmail` and `ServerEmail` for how they are used depending on the type of condition

The new class also needs to implement the `getPipeline()` to return a MongoDB aggregation pipeline which, when applied to the database collection corresponding to the collectionClass, returns a list of alert documents.

The new class must also override `toExecutionAlert()` or `toServerAlert()` which is called on each document returned by the aggregate pipeline.  This method converts the Documents to Alert objects and can also be used for things that would be too cumbersome to do in the aggregate pipeline, like string formatting, data conversions, and complicated arithmetic.

The alert documents for `ServerAlerts` must contain:
 - "server": the name of the server which triggered the alert, or "all" for aggregate conditions
 - "parameters": the name of the parameter which triggered the alert
 - "value": the value which triggered the alert

The alert documents for `ExecutionAlerts` must contain:
 - "executionError.organizationID": the organizationID from the ExecutionRecord
 - "executionError.organizationName": the organization name from the ExecutionRecord
 - "executionError.scenarioID": the scenario ID from the ExecutionRecord
 - "executionError.scenarioName": the scenario name from the ExecutionRecord
 - "executionError.sourceID": the source ID from the ExecutionRecord error
 - "executionError.sourceName": the source name from the ExecutionRecord error
 - "executionError.error": the error message from ExecutionRecord error

Additionally, while pipelines for `ServerAlerts` conditions are always applied as-is (via `Condition.check()`), pipelines for `ExecutionAlerts` conditions should work whether they are applied as-is or with a prepended ScenarioID filter.

##### Testing Conditions

A tool, SmartzoneMonitorConditionRunner (~/solartech/java/test/util/SmartzoneMonitorConditionRunner.java), runs `check()` for a Condition sub-class specified on the command line.  It can be run via the smartzone_monitor_condition_runner.sh script.  This can be used to run conditions against live data which is useful when developing or troubleshooting conditions.

Each condition should also have a unit test in ~/solartech/java/test/com/solartechnology/monitor which exercises the condition against test data in a test database.  The unit test should be added to the test_smartzone_monitor_conditions.sh script which runs all condition unit tests.
