
# HOW TO ADD A NEW ASSET

### Overview
This section will give you and idea about how data gets sent/received between the server-side and client-side portions of our app.

#### Server-side
Upon instantiation of the server-side, the `SolarNetServer.main` method will load units by calling each unit-type's `loadUnits` method, which then queries the DB, fetchs the units, and loads them into `SolarNetServer.units`, `SolarNetServer.unitsBySolarCommID`, `solartech.addAsset` (where solartech represents the current organization), and into the runtime collection specific for that unit-type.

When an asset's detail is modified or a new asset is introduced (of a specific unit-type), the `Asset.createOrModify` method is called which then calls the unit-type specific overridden version of that method. Once the necessary update/create operation has been completed, the `createOrModify` method will call the `SolarNetCollaborator.weCreatedAUnit` method to send the message to update every server by using the `SolarNetCollaborator.sendToEveryoneElse` method.

Each unit-type will have a specific class that will handle communications with the actual/physical unit or an API. When any of the unit's attribute changes, this class will update the unit/asset by calling all the necessary methods. For example, if we are trying to update the battery voltage value of an asset, the asset-type method related to the battery voltage will be called, and this method will then call the `sendToListeners` method triggering the action listener on the client-side `MessageBoardManager` extension for that unit-type. The `MessageBoardManager` will then call the method related to that attribute (battery voltage in this case) and update the related power-unit and consequently the item representing it in the unit-list on the unit-type power-unit page as needed.

#### Client-side
When the client-side gets initialized, the `CommandCenter` is going to create the full UI, including the different unit pages (one for each unit-type), and populate the different pages with the necessary data fetched from the server-side. The units get fetched when creating the menu-bar (`CommandCenter.createMenuBar`) and exchanging a `MsgGetUnitList` message (turned into a `MsgUnitList` message in the `ClientConnection.MySolarNetControlChannel.getUnitList` method) with the server-side (by calling `SolarNetServer.units.values()` - also in the `ClientConnection.MySolarNetControlChannel.getUnitList` method). Each power-unit will be created from a unit, converted to the related power-unit, and then added to that unit-type power-unit page.

When a unit gets created/edited from the UI (by a user), the new/edit dialog for that specific power-unit page opens to collect the unit's data and return it as `UnitData`. This `UnitData` is then used to create a `MsgCreateUnit` (in the `CommandCenter.createNewUnit` - if we are creating a unit - or `CommandCenter.updateUnit` - if we are editing a unit) which is then handled by the `SolarNetControlProtocol`. The `SolarNetControlProtocol` will then send, receive, and dispatch the message, invoke the `SolarNetControlPacketHandlr.createUnit` , and then pass the message to the `ClientConnection.MySolarNetControlChannel`. The `ClientConnection.MySolarNetControlChannel` will call the `Asset.createOrModify` method and dispatch the message to the specific asset-type's `createOrModify` method which then updates the server-side runtime collection for that asset-type and the servers.



### LIST OF CHANGES (divided by file/class and directories)

###### _Notes:_
 - _When creating the necessary classes and introducing the required code changes listed below, replace `<NAME_OF_THE_NEW_ASSET>` (and any similar reference) with whatever the new asset's name is while maintaining the naming-standards used with previous assets._
 - _The following list is in a recommended order of execution that would most likely help avoid errors/bugs, but it's not mandatory, and the changes can be made however the developer feels more comfortable with._
 - _If the standard directory has been used for the [solartech](~/solartech) repo (`~/solartech`), the following links (at least the ones to already existing classes) can be used to open such files directly._


1) #### Initial independent additions:
	- ##### _[images](~solartech/java/images/)_
		- Add the new asset's icon (as a `.png` file) in this directory. This will be the symbol representing the new asset in our application (in map-view). The icon can be added as `icon_<NAME_OF_THE_NEW_ASSET>.png` (use the other assets' icons as a reference for the naming-standards to follow).

	- ##### _[AssetTypes.java](~solartech/java/src/com/solartechnology/util/AssetTypes.java)_
		- Add a new static constant (as `int`) that will be representing the new asset-type.

	- ##### _[<NAME_OF_THE_NEW_ASSET>Data.java](~solartech/java/src/com/solartechnology/solarnet/<NAME_OF_THE_NEW_ASSET>Data.java)_
		- Create the new `<NAME_OF_THE_NEW_ASSET>Data.java` class for the new asset (use the [ForeignSensorData.java](~solartech/java/src/com/solartechnology/solarnet/ForeignSensorData.java) class as a reference).

	- ##### _[<NAME_OF_THE_NEW_ASSET>Info.java](~solartech/java/src/com/solartechnology/solarnet/<NAME_OF_THE_NEW_ASSET>Info.java)_
		- Create the new `<NAME_OF_THE_NEW_ASSET>Info.java` class for the new asset (use the [ForeignSensorInfo.java](~solartech/java/src/com/solartechnology/solarnet/ForeignSensorInfo.java) class as a reference).

	- ##### Add any new class necessary to define/parse specific properties/attributes of the new asset that cannot simply be identified by a variable within the new asset's class.


2) #### Server-side additions:
	- ##### _[<NAME_OF_THE_NEW_ASSET>.java](~solartech/java/src/com/solartechnology/solarnet/<NAME_OF_THE_NEW_ASSET>.java)_
		- Create a new class as `<NAME_OF_THE_NEW_ASSET>.java` that extends [Asset.java](~solartech/java/src/com/solartechnology/solarnet/Asset.java).
		- Add/modify/override the
		    - `createUnit`
		    - `loadUnits`
		    - `createOrModify`

            methods, along with any other method, as needed for the new asset (use the [ForeignSensor.java](~solartech/java/src/com/solartechnology/solarnet/ForeignSensor.java) class as a reference).
		- Add debugging options as needed (use the [ForeignSensor.java](~solartech/java/src/com/solartechnology/solarnet/ForeignSensor.java) class as a reference).

	- ##### _[Asset.java](~solartech/java/src/com/solartechnology/solarnet/Asset.java)_
		- Add all the necessary unimplemented/abstract methods that are specific to the new asset that will be implemented within the new asset's class.
		- Add an `if-clause` checking for the asset-type and calling the new asset's implementation of the `createOrModify` method (called on the server-side when an asset of the new type needs to be created/modified).
		- Add the new asset-specific methods to all the other classes that extend [Asset.java](~solartech/java/src/com/solartechnology/solarnet/Asset.java).


3) #### Data-exchange (server-side to client and vice-versa) related changes:
	- ##### _[UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java)_
		- Add the necessary properties/variables within the [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) class that are specific to the new asset and that will need to be saved in a [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) instance.
		- Add the necessary static keys related to the above mentioned new properties/variables that will be used when loading all [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) props into an `HashMap` or getting the data from one.
		- Add the new props with their value-assignment within the
			- `UnitData(UnitData unitData)`
			- `UnitData(HashMap<String, String> values)`
			- `HashMap<String, String> getAsHashMap()`
	
			methods following the same logic as other props already there from other asset-types.

	- ##### _[MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java)_
		- Add the necessary properties/variables within the [MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java) class that are specific to the new asset and that will need to be saved in a [MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java) instance (should be the same ones added in the [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) class).
		- Add the new props with their value-assignment within the
			- `MsgCreateUnit(String organization, UnitData d)`
			- `copyInto(UnitData u)`
			- `HashMap<String, String> getAsHashMap()`

			methods following the same logic as other props already there from other asset-types.

	- ##### _[NewCmsUnitDialog.java](~solartech/java/src/com/solartechnology/commandcenter/NewCmsUnitDialog.java)_
		- Add the necessary java-swing components (JtextField, JButton, JComboBox, etc.) as needed/necessary to collect the proper data for the new asset-type unit from the user (should be similar/related to the ones added in the [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) class).
		- Add the necessary String-variable to collect the data that will be entered in the above mentioned new java-swing components.
		- Within the

		    ``` java
            NewCmsUnitDialog(JFrame parent, UnitData data, boolean solarnet, boolean commandCenter, boolean messageBoard, boolean arrowBoard, MsgOrganizationList.Organization[] organizations, String currentOrganization)
            ```

            method, add, for each new java-swing component, an `if-clause` checking for both the java-swing component instance and the related String-variable not being `null` and calling the `setText` method on the java-swing component and assigning it the value of the String-variable (use the `if-clauses` implemented for other asset-types' related components as a reference).
		- Modify the

            ``` java
            createGui(JFrame parent, boolean solarnet, boolean commandCenter, boolean messageBoard, boolean arrowBoard, Organization[] organizations, String currentOrganization, int assetType)
            ```

            method as needed to add the new java-swing component to the dialog along with the required `addActionListener` to them as/if needed.
		- Within the `actionPerformed(ActionEvent ev)` method, add, for each new java-swing component, an `if-clause` checking that the component instance is not `null` and instanciating the related String-variable to the value of that component by calling its `getText` method (or a similar method depending on the Java-swing component type - use the `if-clauses` implemented for other asset-types' related components as a reference).
		- Within the `getNewUnit()` method
			- Add, for each new String-variable (related to the new java-swing components), an `if-clause` checking that the String-variable instance is not `null` and instanciating the related [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) prop to the value of that String-variable (use the `if-clauses` implemented for other asset-types' related components as a reference).
			- Add, for each new String-variable (related to the new java-swing components), a call to the `HashMap<String, String> h`'s `put` method to add the String-variable's value to it using the related [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) static key as the `String` key for the `HashMap`.

	- ##### _[Msg<NAME_OF_THE_NEW_ASSET>Data.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/Msg<NAME_OF_THE_NEW_ASSET>Data.java)_
		- Create the new `Msg<NAME_OF_THE_NEW_ASSET>Data.java` class for the new asset (use the [MsgForeignSensorData.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgForeignSensorData.java) class as a reference).


4) #### Server-side changes that depend on previous additions:
	- ##### _[SolarNetControlPacketHandler.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/SolarNetControlPacketHandler.java)_
		- Add the unimplemented/abstract `<NAME_OF_THE_NEW_ASSET>Data(Msg<NAME_OF_THE_NEW_ASSET>Data msg)` method implemented and used within the `Msg<NAME_OF_THE_NEW_ASSET>Data` class.

	- ##### _[SolarNetServer.java](~solartech/java/src/com/solartechnology/solarnet/SolarNetServer.java)_
		- Add a new `ConcurrentHashMap` that will represent the runtime collection of the assets of the new asset-type.
		- Within the `initServer(String dbSuffix, String rootPW)` method, add an `if-clause` checking for the `activeServer` variable to be `true` and calling the new asset's implementation of the static `loadUnits` method (use the

			``` java
			if(activeServer)
			```

			implemented for other asset-types as a reference).
		- Within the `makeDatabaseConnection()` method, add

			``` java
			morphia.map(<NAME_OF_THE_NEW_ASSET>Info.class);
			```

			along with the mapping of the other `*Info.class` classes.
		- Within the `getAssetBySolarCommID(String solarCommID)` method, add an `if-clause` checking for the `Asset a` not being `null` and assigning the correct asset-type to it by calling its implementation of the static `findBySolarCommID` method (use the

			``` java
			if(a == null)
			```

			implemented for other asset-types as a reference).


5) #### Client-side additions:
	- ##### _[PowerUnit<NAME_OF_THE_NEW_ASSET>.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit<NAME_OF_THE_NEW_ASSET>.java)_
		- Create a new class as `PowerUnit<NAME_OF_THE_NEW_ASSET>.java` that extends [PowerUnit.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit.java).
		- Add/modify/override methods as needed for the new asset's power-unit (use the [PowerUnitForeignSensor.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitForeignSensor.java) class as a reference).
		- Ensure to implement/create a private `Manager` class (that extends the [MessageBoardManager.java](~solartech/java/src/com/solartechnology/commandcenter/MessageBoardManager.java) class) and to implement the necessary methods within it that will collect the new asset-data (from messages or values) coming from the server-side (see methods
		    - `batteryVoltage(double voltage)`
		    - `batteryInfo(MsgBatteryInfo msg)`

            in the [PowerUnitForeignSensor.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitForeignSensor.java) class as an example).
		- Add debugging options as needed (use the [PowerUnitForeignSensor.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitForeignSensor.java) class as a reference).
		- Modify the `createGui()` method as needed to add/show the desired new asset's props/data.

	- ##### _[PowerUnit<NAME_OF_THE_NEW_ASSET>Page.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit<NAME_OF_THE_NEW_ASSET>Page.java)_
		- Create a new class as `PowerUnit<NAME_OF_THE_NEW_ASSET>Page.java` that extends [PowerUnitPage.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitPage.java).
		- Add/modify/override the 
		    - `createUnit`
		    - `editUnit`
		    - `deleteUnit`
		    - `addUnitToInterface`
		    - `selectUnit`
		    - `setVisibleUnits`

	        methods, along with any other method, as needed for the new asset's power-unit-page (use the [PowerUnitForeignSensorsPage.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitForeignSensorsPage.java) class as a reference).
		- Add debugging options as needed (use the [PowerUnitForeignSensorsPage.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitForeignSensorsPage.java) class as a reference).

	- ##### _[PowerUnit.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit.java)_
		- Add all the necessary unimplemented/abstract methods that are specific to the new asset-type power-unit implemented within the `PowerUnit<NAME_OF_THE_NEW_ASSET>` class.
		- Add the new asset-type power-unit specific methods to all the other classes that extend [PowerUnit.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit.java).

	- ##### _[PowerUnitPage.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitPage.java)_
		- Add all the necessary unimplemented/abstract methods that are specific to the new asset-type power-unit page implemented within the `PowerUnit<NAME_OF_THE_NEW_ASSET>Page` class.
		- Add the new asset-type power-unit page specific methods to all the other classes that extend [PowerUnitPage.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnitPage.java).

	- ##### _[MessageBoardManager.java](~solartech/java/src/com/solartechnology/commandcenter/MessageBoardManager.java)_
		- Add the unimplemented/abstract `<NAME_OF_THE_NEW_ASSET>UnitData(Msg<NAME_OF_THE_NEW_ASSET>Data msg)` method implemented and used within the `Manager` private class (extension of [MessageBoardManager.java](~solartech/java/src/com/solartechnology/commandcenter/MessageBoardManager.java) class) within the new `PowerUnit<NAME_OF_THE_NEW_ASSET>` class.

	- ##### _[MessageBoardCommunicator.java](~solartech/java/src/com/solartechnology/net/MessageBoardCommunicator.java)_
		- Add the unimplemented/abstract `request<NAME_OF_THE_NEW_ASSET>UnitData()` method used within the `makeInitialRequests` method within the new `PowerUnit<NAME_OF_THE_NEW_ASSET>` class.


6) #### Client-side changes that depend on previous additions:
	- ##### _[CommandCenter.java](~solartech/java/src/com/solartechnology/commandcenter/CommandCenter.java)_
		- Add an instance of the new `PowerUnit<NAME_OF_THE_NEW_ASSET>Page` that will be used within `CommandCenter` to manage the new asset's power-units page.
		- Within the `MySolarnetControlPacketHandler` private class (that extends [SolarNetControlPacketHandler.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/SolarNetControlPacketHandler.java)), make the following additions/modifications:
			- Within the `MySolarnetControlPacketHandler.createUnit(MsgCreateUnit msg)` method
				- Add an `if-clause` checking for the type of unit and calling the new `PowerUnit<NAME_OF_THE_NEW_ASSET>Page`'s implementation of the `addUnitToInterface` method to add the unit to the correct power-unit-page (use the

					``` java
					if(unit instanceof PowerUnit<NAME_OF_THE_NEW_ASSET>)
					```

					implemented for other asset-types' power-unit as a reference).
				- Add an `if-clause` checking for the type of unit and calling the new `PowerUnit<NAME_OF_THE_NEW_ASSET>Page`'s implementation of the `unitDataChanged` method (use the

					``` java
					if(unit instanceof PowerUnit<NAME_OF_THE_NEW_ASSET>)
					```

					implemented for other asset-types' power-unit as a reference).
			- Within the `MySolarnetControlPacketHandler.createNewUnit(MsgCreateUnit msg)` method, add an `else-if-clause` checking for the type of asset provided within the [MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java) instance passed in and instanciating the `unit` (instance of the [PowerUnit.java](~solartech/java/src/com/solartechnology/commandcenter/PowerUnit.java) class) to an instance of the new `PowerUnit<NAME_OF_THE_NEW_ASSET>` (use the

				``` java
				else if(msg.assetType == AssetTypes.<NAME_OF_THE_NEW_ASSET>)
				```

				implemented for other asset-types as a reference).
			- Within the `MySolarnetControlPacketHandler.unitList(MsgUnitList msg)` method
				- Add a new `ArrayList<T>` (where `T` is the new `PowerUnit<NAME_OF_THE_NEW_ASSET>`) that will be collecting all the power-units of type `PowerUnit<NAME_OF_THE_NEW_ASSET>` found in the instance of [MsgUnitList.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgUnitList.java) passed in as a parameter.
				- Add an `else-if-clause` checking for the type of power-unit provided in `MsgCreateUnit.units` and calling the `add` method on the `ArrayList<PowerUnit<NAME_OF_THE_NEW_ASSET>>` to add the unit to the related-type collection (use the

					``` java
					else if(unit instanceof PowerUnit<NAME_OF_THE_NEW_ASSET>)
					```

					implemented for other asset-types' power-unit as a reference).
				- Add a call to the `PowerUnit<NAME_OF_THE_NEW_ASSET>Page`'s `setUnits()` method to add the full collection of new asset-type power-units to the related page (add this where the same operation is performed for the other asset-types' power-unit-pages) and add a new value within the `benchmarks` array

					``` java
					// `XX` is whatever number needed - increment from the current one
					benchmarks[XX] = System.nanoTime();
					```

					(make sure to increment the size of the `benchmarks` array as needed at its instantiation).
		- Within the `createGUIComponentsIfNecessary()` method
			- Instanciate the new `PowerUnit<NAME_OF_THE_NEW_ASSET>Page` instance (use the

				``` java
				new PowerUnit<NAME_OF_THE_NEW_ASSET>Page(controlCenter, false, localLibraries);
				```

				implemented for other asset-types as a reference).
			- Add an `if-clause` checking that the new `PowerUnit<NAME_OF_THE_NEW_ASSET>Page` instance is not `null` and calling

				``` java
				mainTabs.add(TR.get("<NAME_OF_THE_NEW_ASSET_POWER_UNIT_PAGE>"), PowerUnit<NAME_OF_THE_NEW_ASSET>Page instance);
				```

				(use the same call to the `mainTabs.add(...)` implemented for other asset-types' power-unit-page as a reference).
		- Within the `addConnectionToSolarnetServer(String preferred_server)` method
			- Add a call to the `PowerUnit<NAME_OF_THE_NEW_ASSET>Page`'s `adaptToAccount` method (use the calls to

				``` java
				PowerUnit<NAME_OF_THE_NEW_ASSET>Page.adaptToAccount(solarnetAccount);
				```

				implemented for other asset-types' powe-unit-page).
			- Add an `if-clause` checking for the type of unit (as assigned to the new [MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java) instance `cu`) and instantiating the [MsgCreateUnit.java](~solartech/java/src/com/solartechnology/protocols/solarnetcontrol/MsgCreateUnit.java)'s props associated with the new asset with the values of those same props found in the [UnitData.java](~solartech/java/src/com/solartechnology/commandcenter/UnitData.java) instance passed in as the parameter (use the

				``` java
				if(cu.assetType == AssetTypes.FOREIGN_SENSOR)
				```

				as a reference).

